; ataio.asm
;
; functions for low level ATA/ATAPI I/O
;
; Copyright (C) 2000, Suzhe. See file COPYING and CREDITS for details.
;
; This file is based on the Public Domain software -- 
; ATA LOW LEVEL I/O DRIVER by Hale Landis (hlandis@ibm.net)
;

%include "ataio.h"


;=============================================================
;
; Host adapter base addresses.
;
;=============================================================

pio_base_addr1 dw 0x1f0
pio_base_addr2 dw 0x3f0

pio_reg_addrs times 10 dw 0

pio_last_write times 10 db 0
pio_last_read  times 10 db 0


;=============================================================
;
; Set the host adapter i/o base addresses.
;
; input:
;      ax = base 1
;      bx = base 2
; output:
;      none
;=============================================================

pio_set_iobase_addr:
        push ax
        push bx

        mov [pio_base_addr1], ax
        mov [pio_base_addr2], bx
        mov [pio_reg_addrs + CB_DATA*2], ax  ; 0
        inc ax
        mov [pio_reg_addrs + CB_FR*2], ax    ; 1
        inc ax
        mov [pio_reg_addrs + CB_SC*2], ax    ; 2
        inc ax
        mov [pio_reg_addrs + CB_SN*2], ax    ; 3
        inc ax
        mov [pio_reg_addrs + CB_CL*2], ax    ; 4
        inc ax
        mov [pio_reg_addrs + CB_CH*2], ax    ; 5
        inc ax
        mov [pio_reg_addrs + CB_DH*2], ax    ; 6
        inc ax
        mov [pio_reg_addrs + CB_CMD*2], ax   ; 7
        add bx, 6
        mov [pio_reg_addrs + CB_DC*2], bx    ; 8
        inc bx
        mov [pio_reg_addrs + CB_DA*2], bx    ; 9

        pop bx
        pop ax
        ret

;=========================================================
; pio_inbyte
;
; input:
;      dx = addr
; output:
;      al = data
;=========================================================
pio_inbyte:
        push bx
        push dx

        mov bx, dx
        shl bx, 1
        mov dx, [pio_reg_addrs + bx]
        in  al, dx

        shr bx, 1
        mov [pio_last_read + bx], al

        pop dx
        pop bx

        ret

;=========================================================
; pio_outbyte
;
; input:
;      al = data
;      dx = addr
; output:
;      none
;=========================================================
pio_outbyte:
        push bx
        push dx

        mov bx, dx
        shl bx, 1
        mov dx, [pio_reg_addrs + bx]
        out dx, al

        shr bx, 1
        mov [pio_last_write + bx], al

        pop dx
        pop bx

        ret

;=========================================================
; pio_inword
;
; input:
;      dx = addr
; output:
;      ax = data
;=========================================================
pio_inword:
        push bx
        push dx

        mov bx, dx
        shl bx, 1
        mov dx, [pio_reg_addrs + bx]
        in  ax, dx

        pop dx
        pop bx

        ret

;=========================================================
; pio_outword
;
; input:
;      ax = data
;      dx = addr
; output:
;      none
;=========================================================
pio_outword:
        push bx
        push dx

        mov bx, dx
        shl bx, 1
        mov dx, [pio_reg_addrs + bx]
        out dx, ax

        pop dx
        pop bx

        ret

;=========================================================
; pio_rep_inword
;
; input:
;      cx = count
;      dx = addr
;      es:di -> buffer
; output:
;      none
;=========================================================
pio_rep_inword:
        push cx
        push bx
        push dx
        push di

        mov bx, dx
        shl bx, 1
        mov dx, [pio_reg_addrs + bx]

        cld
        rep insw

        pop di
        pop dx
        pop bx
        pop cx

        ret


;=========================================================
; pio_rep_outword
;
; input:
;      cx = count
;      dx = addr
;      ds:si -> buffer
; output:
;      none
;=========================================================
pio_rep_outword:
        push cx
        push bx
        push dx
        push si

        mov bx, dx
        shl bx, 1
        mov dx, [pio_reg_addrs + bx]

        cld
        rep outsw

        pop si
        pop dx
        pop bx
        pop cx

        ret


; end ataio.asm
